/*!--------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/(async function(){performance.mark("code/didStartRenderer");const c=window.vscode,d=c.process;function p(o){performance.mark("code/willShowPartsSplash");let e=o.partsSplash;e&&(o.autoDetectHighContrast&&o.colorScheme.highContrast?(o.colorScheme.dark&&e.baseTheme!=="hc-black"||!o.colorScheme.dark&&e.baseTheme!=="hc-light")&&(e=void 0):o.autoDetectColorScheme&&(o.colorScheme.dark&&e.baseTheme!=="vs-dark"||!o.colorScheme.dark&&e.baseTheme!=="vs")&&(e=void 0));let t="vs-dark",s="#1E1E1E",l="#CCCCCC";e?(t=e.baseTheme,s=e.baseTheme==="vs"?e.colorInfo.background??e.colorInfo.editorBackground:e.colorInfo.editorBackground??e.colorInfo.background,l=e.colorInfo.foreground??l):o.autoDetectHighContrast&&o.colorScheme.highContrast?o.colorScheme.dark?(t="hc-black",s="#000000",l="#FFFFFF"):(t="hc-light",s="#FFFFFF",l="#000000"):o.autoDetectColorScheme&&(o.colorScheme.dark?(t="vs-dark",s="#1E1E1E",l="#CCCCCC"):(t="vs",s="#F3F3F3",l="#000000"));const a=document.createElement("style");a.className="initialShellColors",window.document.head.appendChild(a),a.textContent=`body { background-color: ${s}; color: ${l}; margin: 0; padding: 0; }`,typeof e?.zoomLevel=="number"&&typeof c?.webFrame?.setZoomLevel=="function"&&c.webFrame.setZoomLevel(e.zoomLevel);const r=document.createElement("div");r.id="monaco-parts-splash",r.className=t??"vs-dark",window.document.body.appendChild(r),performance.mark("code/didShowPartsSplash")}async function m(o){const e=await h();o?.beforeImport?.(e);const{enableDeveloperKeybindings:t,removeDeveloperKeybindingsAfterLoad:s,developerDeveloperKeybindingsDisposable:l,forceDisableShowDevtoolsOnError:a}=f(e,o);b(e);const r=new URL(`${w(e.appRoot,{isWindows:d.platform==="win32",scheme:"vscode-file",fallbackAuthority:"vscode-app"})}/out/`);globalThis._VSCODE_FILE_ROOT=r.toString(),S(e,r);try{let n;d.env.VSCODE_DEV&&globalThis._VSCODE_USE_RELATIVE_IMPORTS?n="./sessions.desktop.main.js":n=new URL("vs/sessions/sessions.desktop.main.js",r).href;const i=await import(n);return l&&s&&l(),{result:i,configuration:e}}catch(n){throw v(n,t&&!a),n}}async function h(){const o=setTimeout(()=>{console.error("[resolve window config] Could not resolve window configuration within 10 seconds, but will continue to wait...")},1e4);performance.mark("code/willWaitForWindowConfig");const e=await c.context.resolveConfiguration();return performance.mark("code/didWaitForWindowConfig"),clearTimeout(o),e}function f(o,e){const{forceEnableDeveloperKeybindings:t,disallowReloadKeybinding:s,removeDeveloperKeybindingsAfterLoad:l,forceDisableShowDevtoolsOnError:a}=typeof e?.configureDeveloperSettings=="function"?e.configureDeveloperSettings(o):{forceEnableDeveloperKeybindings:!1,disallowReloadKeybinding:!1,removeDeveloperKeybindingsAfterLoad:!1,forceDisableShowDevtoolsOnError:!1},n=!!(!!d.env.VSCODE_DEV||t);let i;return n&&(i=u(s)),{enableDeveloperKeybindings:n,removeDeveloperKeybindingsAfterLoad:l,developerDeveloperKeybindingsDisposable:i,forceDisableShowDevtoolsOnError:a}}function u(o){const e=c.ipcRenderer,t=function(n){return[n.ctrlKey?"ctrl-":"",n.metaKey?"meta-":"",n.altKey?"alt-":"",n.shiftKey?"shift-":"",n.keyCode].join("")},s=d.platform==="darwin"?"meta-alt-73":"ctrl-shift-73",l="123",a=d.platform==="darwin"?"meta-82":"ctrl-82";let r=function(n){const i=t(n);i===s||i===l?e.send("vscode:toggleDevTools"):i===a&&!o&&e.send("vscode:reloadWindow")};return window.addEventListener("keydown",r),function(){r&&(window.removeEventListener("keydown",r),r=void 0)}}function b(o){globalThis._VSCODE_NLS_MESSAGES=o.nls.messages,globalThis._VSCODE_NLS_LANGUAGE=o.nls.language;let e=o.nls.language||"en";e==="zh-tw"?e="zh-Hant":e==="zh-cn"&&(e="zh-Hans"),window.document.documentElement.setAttribute("lang",e)}function v(o,e){e&&c.ipcRenderer.send("vscode:openDevTools"),console.error(`[uncaught exception]: ${o}`),o&&typeof o!="string"&&o.stack&&console.error(o.stack)}function w(o,e){let t=o.replace(/\\/g,"/");t.length>0&&t.charAt(0)!=="/"&&(t=`/${t}`);let s;return e.isWindows&&t.startsWith("//")?s=encodeURI(`${e.scheme||"file"}:${t}`):s=encodeURI(`${e.scheme||"file"}://${e.fallbackAuthority||""}${t}`),s.replace(/#/g,"%23")}function S(o,e){if(!globalThis._VSCODE_DISABLE_CSS_IMPORT_MAP&&Array.isArray(o.cssModules)&&o.cssModules.length>0){performance.mark("code/willAddCssLoader"),globalThis._VSCODE_CSS_LOAD=function(r){const n=document.createElement("link");n.setAttribute("rel","stylesheet"),n.setAttribute("type","text/css"),n.setAttribute("href",r),window.document.head.appendChild(n)};const t={imports:{}};for(const r of o.cssModules){const n=new URL(r,e).href,i=`globalThis._VSCODE_CSS_LOAD('${n}');
`,y=new Blob([i],{type:"application/javascript"});t.imports[n]=URL.createObjectURL(y)}const s=window.trustedTypes?.createPolicy("vscode-bootstrapImportMap",{createScript(r){return r}}),l=JSON.stringify(t,void 0,2),a=document.createElement("script");a.type="importmap",a.setAttribute("nonce","0c6a828f1297"),a.textContent=s?.createScript(l)??l,window.document.head.appendChild(a),performance.mark("code/didAddCssLoader")}}const{result:g,configuration:D}=await m({configureDeveloperSettings:function(o){return{forceDisableShowDevtoolsOnError:typeof o.extensionTestsPath=="string"||o["enable-smoke-test-driver"]===!0,forceEnableDeveloperKeybindings:Array.isArray(o.extensionDevelopmentPath)&&o.extensionDevelopmentPath.length>0,removeDeveloperKeybindingsAfterLoad:!0}},beforeImport:function(o){p(o),Object.defineProperty(window,"vscodeWindowId",{get:()=>o.windowId}),window.requestIdleCallback(()=>{const e=document.createElement("canvas");e.getContext("2d")?.clearRect(0,0,e.width,e.height),e.remove()},{timeout:50}),performance.mark("code/willLoadWorkbenchMain")}});performance.mark("code/didLoadWorkbenchMain"),g.main(D)})();

//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-221e0a382c0be3a673a4e4cab0601344a0b3de3a/core-vs-sessions-electron-browser-sessions.js.map
