"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.TunnelProgress = exports.TunnelConstraints = exports.TunnelServiceProperties = exports.TunnelProtocol = exports.TunnelHeaderNames = exports.TunnelEvent = exports.TunnelEndpoint = exports.TunnelConnectionMode = exports.TunnelAccessScopes = exports.TunnelAccessControlEntryType = exports.TunnelAccessControlEntry = exports.TunnelAccessControl = void 0;
var tunnelAccessControl_1 = require("./tunnelAccessControl");
Object.defineProperty(exports, "TunnelAccessControl", { enumerable: true, get: function () { return tunnelAccessControl_1.TunnelAccessControl; } });
var tunnelAccessControlEntry_1 = require("./tunnelAccessControlEntry");
Object.defineProperty(exports, "TunnelAccessControlEntry", { enumerable: true, get: function () { return tunnelAccessControlEntry_1.TunnelAccessControlEntry; } });
var tunnelAccessControlEntryType_1 = require("./tunnelAccessControlEntryType");
Object.defineProperty(exports, "TunnelAccessControlEntryType", { enumerable: true, get: function () { return tunnelAccessControlEntryType_1.TunnelAccessControlEntryType; } });
var tunnelAccessScopes_1 = require("./tunnelAccessScopes");
Object.defineProperty(exports, "TunnelAccessScopes", { enumerable: true, get: function () { return tunnelAccessScopes_1.TunnelAccessScopes; } });
var tunnelConnectionMode_1 = require("./tunnelConnectionMode");
Object.defineProperty(exports, "TunnelConnectionMode", { enumerable: true, get: function () { return tunnelConnectionMode_1.TunnelConnectionMode; } });
var tunnelEndpoint_1 = require("./tunnelEndpoint");
Object.defineProperty(exports, "TunnelEndpoint", { enumerable: true, get: function () { return tunnelEndpoint_1.TunnelEndpoint; } });
var tunnelEvent_1 = require("./tunnelEvent");
Object.defineProperty(exports, "TunnelEvent", { enumerable: true, get: function () { return tunnelEvent_1.TunnelEvent; } });
var tunnelHeaderNames_1 = require("./tunnelHeaderNames");
Object.defineProperty(exports, "TunnelHeaderNames", { enumerable: true, get: function () { return tunnelHeaderNames_1.TunnelHeaderNames; } });
var tunnelProtocol_1 = require("./tunnelProtocol");
Object.defineProperty(exports, "TunnelProtocol", { enumerable: true, get: function () { return tunnelProtocol_1.TunnelProtocol; } });
var tunnelServiceProperties_1 = require("./tunnelServiceProperties");
Object.defineProperty(exports, "TunnelServiceProperties", { enumerable: true, get: function () { return tunnelServiceProperties_1.TunnelServiceProperties; } });
var tunnelConstraints_1 = require("./tunnelConstraints");
Object.defineProperty(exports, "TunnelConstraints", { enumerable: true, get: function () { return tunnelConstraints_1.TunnelConstraints; } });
var tunnelProgress_1 = require("./tunnelProgress");
Object.defineProperty(exports, "TunnelProgress", { enumerable: true, get: function () { return tunnelProgress_1.TunnelProgress; } });
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-221e0a382c0be3a673a4e4cab0601344a0b3de3a/node_modules-@microsoft-dev-tunnels-contracts-index.js.map