"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
// Generated from ../../../cs/src/Contracts/ErrorCodes.cs
/* eslint-disable */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorCodes = void 0;
/**
 * Error codes for ErrorDetail.Code and `x-ms-error-code` header.
 */
var ErrorCodes;
(function (ErrorCodes) {
    /**
     * Operation timed out.
     */
    ErrorCodes["Timeout"] = "Timeout";
    /**
     * Operation cannot be performed because the service is not available.
     */
    ErrorCodes["ServiceUnavailable"] = "ServiceUnavailable";
    /**
     * Internal error.
     */
    ErrorCodes["InternalError"] = "InternalError";
})(ErrorCodes = exports.ErrorCodes || (exports.ErrorCodes = {}));
//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-221e0a382c0be3a673a4e4cab0601344a0b3de3a/node_modules-@microsoft-dev-tunnels-contracts-errorCodes.js.map