// separate file so I stop getting nagged in vim about deprecated API.
export const umask = () => process.umask();//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-221e0a382c0be3a673a4e4cab0601344a0b3de3a/node_modules-tar-dist-esm-process-umask.js.map